/*****************************************************************************
* File....: DEMO.PRG                                                         *
* By......: David A Pearson                                                  *
* Function: Demo program for use with the ArjDirectory() function.           *
* Notes...: Compile with switches /n/w                                       *
*****************************************************************************/

#include "box.ch"
#include "misc.ch"
#include "inkey.ch"
#include "arjdir.ch"

#define TB_HEADSEP      ""
#define TB_COLSEP       "  "
#define TB_FOOTSEP      ""

#define COLUMN_STORE    3

/*****************************************************************************
* Function: Main()                                                           *
* Syntax..: Main(<cFileName>) --> NIL                                        *
* Usage...: Entry function for the demo program.                             *
* By......: David A Pearson                                                  *
*****************************************************************************/

function Main(cFileName)
local aArjDir := {},;
      cOldCol := NULL
set date british
if !empty(cFileName := Default(cFileName,NULL))
   if !empty(aArjDir := ArjDirectory(cFileName))
      cOldCol := setcolor(if(iscolor(),"gr+/b,w+/r",setcolor()))
      BrowseDir(aArjDir)
      setcolor(cOldCol)
   else
      ? "It appears that there was some kind of problem!"
      ? "Error code == "+alltrim(str(ArjDirErr()))
   endif
else
   ? "Usage: Demo <ArjFile>"
endif
return(NIL)

/*****************************************************************************
* Function: BrowseDir()                                                      *
* Syntax..: BrowseDir(<aArjDir>) --> NIL                                     *
* Usage...: Browse the structure of the ARJ file.                            *
* By......: David A Pearson                                                  *
*****************************************************************************/

static function BrowseDir(aArjDir)
local oBrowser := BuildBrowse(aArjDir),;
      nKey     := 0                   ,;
      cScreen  := savescreen()        ,;
      nOldRow  := row()               ,;
      nOldCol  := col()
dispbox(oBrowser:nTop-1,oBrowser:nLeft-1,oBrowser:nBottom,oBrowser:nRight+1,B_DOUBLE)
do forever
   dispbegin()
   do while !oBrowser:stable .and. ((nKey := inkey()) == 0)
      oBrowser:stabilize()
   enddo
   dispend()
   if oBrowser:stable
      if oBrowser:hitTop .or. oBrowser:hitBottom
         tone(100,3)
         if oBrowser:hitTop()
            oBrowser:goTop()
         else
            oBrowser:hitBottom()
         endif
         oBrowser:refreshAll()
         loop
      endif
      nKey := inkey(0)
   endif
   do case
      case nKey == K_ESC
           exit
      case BrowseKeys(oBrowser,nKey)
   endcase
enddo
restscreen(,,,,cScreen)
devpos(nOldRow,nOldCol)
return(NIL)

/*****************************************************************************
* Function: BuildBrowse()                                                    *
* Syntax..: BuildBrowse(<aArjDir>) --> oBrowse                               *
* Usage...: Builds the TBrowse object.                                       *
* By......: David A Pearson                                                  *
*****************************************************************************/

static function BuildBrowse(aArjDir)
local oBrowser    := TBrowseNew(4,4,maxrow()-4,maxcol()-4),;
      oColumn     := NIL                                  ,;
      nFNameWidth := 0
aeval(aArjDir,{|x| nFNameWidth := max(nFNameWidth,len(x[ARJ_FNAME]))})
oBrowser:goTopBlock    := {|| Element(1) }
oBrowser:goBottomBlock := {|| Element(len(aArjDir)) }
oBrowser:skipBlock     := {|nMove| SkipArray(nMove,len(aArjDir)) }
oBrowser:colSep        := TB_COLSEP
oBrowser:headSep       := TB_HEADSEP
oBrowser:addColumn(TBColumnNew("File Name" ,{|| padr(aArjDir[Element()][ARJ_FNAME],nFNameWidth) }))
oBrowser:addColumn(TBColumnNew("Date"      ,{|| aArjDir[Element()][ARJ_DATE] }))
oBrowser:addColumn(TBColumnNew("Time"      ,{|| aArjDir[Element()][ARJ_TIME] }))
oBrowser:addColumn(TBColumnNew("Size"      ,{|| aArjDir[Element()][ARJ_ORIG_SIZE] }))
oBrowser:addColumn(TBColumnNew("Compressed",{|| aArjDir[Element()][ARJ_COMP_SIZE] }))
oBrowser:addColumn(TBColumnNew("Host OS"   ,{|| ArjOs(aArjDir[Element()][ARJ_HOST_OS]) }))
oBrowser:addColumn(TBColumnNew("File Type" ,{|| ArjFType(aArjDir[Element()][ARJ_COMP_METHOD]) }))
oBrowser:addColumn(TBColumnNew("Compression Method",{|| ArjCmpMeth(aArjDir[Element()][ARJ_COMP_METHOD]) }))
aeval(oBrowser[COLUMN_STORE],{|oCol| oCol:footSep := TB_FOOTSEP })
return(oBrowser)

/*****************************************************************************
* Function: ArjOs()                                                          *
* Syntax..: ArjOs(<nOSCode>) --> cOSName                                     *
* Usage...: Converts the ARJ OS code into a text name.                       *
* By......: David A Pearson                                                  *
*****************************************************************************/

static function ArjOs(nOS)
return(padr({"MSDOS","PRIMOS","UNIX","AMIGA","MAC-OS","OS/2","APPLE GS",;
             "ATARI ST","NEXT","VAX VMS"}[nOS+1],8))

/*****************************************************************************
* Function: ArjFType()                                                       *
* Syntax..: ArjFType(<nType>) --> cType                                      *
* Usage...: Converts the ARJ file type code into a text name.                *
* By......: David A Pearson                                                  *
*****************************************************************************/

static function ArjFType(nType)
return(padr({"Binary","7-Bit Text","Directory","Volume Label"}[nType+1],12))

/*****************************************************************************
* Function: ArjCmpMeth()                                                     *
* Syntax..: ArjCmpMeth(<nMethod>) --> cMethod                                *
* Usage...: Converts the ARJ method code into a text name.                   *
* By......: David A Pearson                                                  *
*****************************************************************************/

static function ArjCmpMeth(nMethod)
return(padr({"Stored","Compressed Most","Compressed 2","Compressed 3",;
        "Compressed Fastest"}[nMethod+1],18))

/*****************************************************************************
* Function: Element()                                                        *
* Syntax..: Element([<nNewValue>]) --> nOldvalue                             *
* Usage...: Holds the current position in the array being browsed.           *
* By......: David A Pearson                                                  *
*****************************************************************************/

static function Element(nNewValue)
static nElement := 1
if valtype(nNewValue) == "N"
   nElement := nNewValue
endif
return(nElement)

/*****************************************************************************
* Function: SkipArray()                                                      *
* Syntax..: SkipArray(<nMove>,<nLastLine>) --> nCanMove                      *
* Usage...: Skipper function for browsing an array.                          *
* By......: David A Pearson                                                  *
*****************************************************************************/

static function SkipArray(nMove,nLastLine)
local nMoved := 0,;
      nLine  := Element()
if nMove > 0
   if nLine + nMove > nLastLine
      nMoved := nLastLine - nLine
      nLine  := nLastLine
   else
      nMoved := nMove
      nLine  += nMove
   endif
else
   if nLine + nMove < 1
      nMoved := 1 - nLine
      nLine  := 1
   else
      nMoved := nMove
      nLine  += nMove
   endif
endif
Element(nLine)
return(nMoved)

/*****************************************************************************
* Function..: BrowseKeys()                                                   *
* Syntax....: BrowseKeys(<oObject>,<nKey>) --> lKeyProcessed                 *
* By........: David A Pearson                                                *
* Notes.....: Key handler for use with a TBrowse Object.                     *
*****************************************************************************/

function BrowseKeys(oObject,nKey)
local lKeyProcessed := TRUE
do case
   case nKey == K_DOWN
        oObject:down()
   case nKey == K_UP
        oObject:up()
   case nKey == K_LEFT
        if oObject:colCount == 1
           oObject:up()
        else
           oObject:left()
        endif
   case nKey == K_RIGHT
        if oObject:colCount == 1
           oObject:down()
        else
           oObject:right()
        endif
   case nKey == K_PGUP
        oObject:pageUp()
   case nKey == K_PGDN
        oObject:pageDown()
   case nKey == K_CTRL_PGUP
        oObject:goTop()
   case nKey == K_CTRL_PGDN
        oObject:goBottom()
   case nKey == K_CTRL_LEFT
        if oObject:colCount == 1
           oObject:up()
        else
           oObject:panLeft()
        endif
   case nKey == K_CTRL_RIGHT
        if oObject:colCount == 1
           oObject:down()
        else
           oObject:panRight()
        endif
   case nKey == K_HOME
        if oObject:colCount == 1
           oObject:goTop()
        else
           oObject:home()
        endif
   case nKey == K_END
        if oObject:colCount == 1
           oObject:goBottom()
        else
           oObject:end()
        endif
   case nKey == K_CTRL_HOME
        if oObject:colCount == 1
           oObject:goTop()
        else
           oObject:panHome()
        endif
   case nKey == K_CTRL_END
        if oObject:colCount == 1
           oObject:goBottom()
        else
           oObject:panEnd()
        endif
   otherwise
        lKeyProcessed := FALSE
endcase
return(lKeyProcessed)
